# -*- coding: ascii -*-
# $Id$
# $Source$
# $Author$
# $HeadURL$
# $Revision$
# $Date$
'''
    roleplay:

        Implements Roles for Python.

        See the Traits paper[1], Perl 6 Synopsis 12[2] or Perl 6 Apocalypse 12[3]
        for more information.


    author: Ask Solem <askh@opera.com>
    Copyright (c) Ask Solem. Released under the Modified BSD-license, see the
    LICENSE file for a full copy of the license.
     
'''


__version__     = '0.11'
__author__      = 'Ask Solem <askh@opera.com'
__authority__   = 'pypi:ASK'


from roleplay.keyword import *


# Local Variables:
#   mode: cpython
#   cpython-indent-level: 4
#   fill-column: 78
# End:
# vim: expandtab tabstop=4 shiftwidth=4 shiftround
